package com.gic.auth.service.outer.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.gic.auth.entity.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AccountGroupMemberTypeEnum;
import com.gic.auth.dto.*;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.*;
import com.gic.auth.utils.ValidSplitUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.AutoCreatePasswordUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.utils.valid.ValidUtil;

/**
 * @author guojx
 * @date 2019/7/16 6:55 PM
 */
@Service("userApiService")
public class UserApiServiceImpl implements UserApiService {
    @Autowired
    private UserService userService;
    @Autowired
    private UserRoleApiService userRoleApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private UserRoleService userRoleService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private ResourceService resourceService;
    @Autowired
    private RoleService roleService;
    @Autowired
    private AccountGroupRelService accountGroupRelService;
    @Autowired
    private AccountGroupService accountGroupService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Integer> saveUser(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.SaveUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        if (userService.isPhoneRepeat(null, userDTO.getPhoneNumber(), userDTO.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }

        //如果是普通管理员，则需要验证角色权限和资源权限
        if (userDTO.getSuperAdmin().intValue() == 0) {
            if (StringUtils.isBlank(userDTO.getUserRoleIds())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色权限不能为空");
            }
            if (StringUtils.isBlank(userDTO.getUserResourceIds())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源权限不能为空");
            }
        }

        if (userDTO.getSuperAdmin() == null) {
            //默认是普通管理员
            userDTO.setSuperAdmin(0);
        }
        //超级管理员不能创建多条，需要判断控制
        if (userDTO.getSuperAdmin().intValue() == 1) {
            TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
            if (adminUser != null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已存在超级管理员，重复创建");
            }
        }

        if (userDTO.getPasswordType() == null) {
            //默认自定义创建密码类型
            userDTO.setPasswordType(2);
        }

        if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
            userDTO.setPhoneAreaCode("86");
        }
        userDTO.setCreateTime(new Date());
        userDTO.setUpdateTime(new Date());
        userDTO.setStatus(1);
        Integer userId = userService.saveUser(userDTO);

        userRoleService.deleteByUserId(userId);
        userResourceService.deleteByUserId(userId);
        //如果账号分组有数据，则需要维护分组数据
        if (StringUtils.isNotBlank(userDTO.getAccountGroupIds())) {
            accountGroupRelService.deleteByUserId(userId, AccountGroupMemberTypeEnum.ADMIN.getCode());
        }

        ServiceResponse<Integer> response = saveRole(userId, userDTO.getEnterpriseId(), userDTO.getUserRoleIds(),
                userDTO.getUserResourceIds(), userDTO.getAccountGroupIds());
        if (response.isSuccess()) {
            return response;
        } else {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return response;
        }
    }

    @Override
    public ServiceResponse<Integer> saveAdmin(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.SaveUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        if (userService.isPhoneRepeat(null, userDTO.getPhoneNumber(), userDTO.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }
        if (userDTO.getPasswordType() == null) {
            //默认自定义创建密码类型
            userDTO.setPasswordType(2);
        }

        if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
            userDTO.setPhoneAreaCode("86");
        }
        userDTO.setCreateTime(new Date());
        userDTO.setUpdateTime(new Date());
        userDTO.setStatus(1);
        Integer userId = userService.saveUser(userDTO);
        return ServiceResponse.success(userId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse editUser(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.EditUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        TabSysUser tabUser = userService.getUserById(userDTO.getUserId());
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误");
        }
        if (userService.isPhoneRepeat(userDTO.getUserId(), userDTO.getPhoneNumber(), tabUser.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }

        if (userDTO.getSuperAdmin() != null && userDTO.getSuperAdmin().intValue() == 1) {
            //超级官员
            TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
            if (adminUser != null && adminUser.getUserId().intValue() != tabUser.getUserId().intValue()) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已存在超级管理员，参数有误");
            }
        }

        if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
            userDTO.setPhoneAreaCode("86");
        }
        userService.editUser(userDTO);

        userRoleService.deleteByUserId(userDTO.getUserId());
        userResourceService.deleteByUserId(userDTO.getUserId());
        //如果账号分组有数据，则需要维护分组数据
        if (StringUtils.isNotBlank(userDTO.getAccountGroupIds())) {
            accountGroupRelService.deleteByUserId(userDTO.getUserId(), AccountGroupMemberTypeEnum.ADMIN.getCode());
        }

        ServiceResponse response = saveRole(userDTO.getUserId(), userDTO.getEnterpriseId(), userDTO.getUserRoleIds(),
                userDTO.getUserResourceIds(), userDTO.getAccountGroupIds());
        if (response.isSuccess()) {
            return response;
        } else {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return response;
        }
    }

    @Override
    public ServiceResponse<Void> editAdmin(UserDTO userDTO) {
        //超级官员
        TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
        if (adminUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "超级管理员不存在，操作有误");
        }
        if (userDTO.getPasswordType() != null && userDTO.getPasswordType().intValue() == 2) {
            //如果是自定义密码
            if (StringUtils.isBlank(userDTO.getPassword())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "密码为空，操作有误");
            }
            userDTO.setPassword(AutoCreatePasswordUtils.encryptPassword(userDTO.getPassword()));
        } else if (userDTO.getPasswordType() != null && userDTO.getPasswordType().intValue() == 1) {
            //如果是自动生成
            userDTO.setPassword(AutoCreatePasswordUtils.autoCreatePassword(8));
        }
        userDTO.setUserId(adminUser.getUserId());
        userService.editUser(userDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> getUserById(Integer userId) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误");
        }
        UserDTO result = EntityUtil.changeEntityNew(UserDTO.class, tabUser);
        //角色集
        List<UserRoleDTO> userRoleDTOList = userRoleService.listUserRoleByUserId(userId);
        StringBuilder userRoleStr = new StringBuilder();
        for (UserRoleDTO userRoleDTO : userRoleDTOList) {
            userRoleStr.append(userRoleDTO.getRoleId()).append(",");
        }
        String userRoleResult = userRoleStr.toString();
        if (StringUtils.isBlank(userRoleResult)) {
            result.setUserRoleIds("");
        } else {
            result.setUserRoleIds(userRoleResult.substring(0, userRoleResult.length() - 1));
        }
        //资源集
        List<UserResourceDTO> userResourceDTOList = userResourceService.listUserResourceByUserId(userId);
        StringBuilder userResourceStr = new StringBuilder();
        for (UserResourceDTO userResourceDTO : userResourceDTOList) {
            userResourceStr.append(userResourceDTO.getResourceId()).append(",");
        }
        String userResourceResult = userResourceStr.toString();
        if (StringUtils.isBlank(userResourceResult)) {
            result.setUserResourceIds("");
        } else {
            result.setUserResourceIds(userResourceResult.substring(0, userResourceResult.length() - 1));
        }
        //账号分组
        List<TabSysAccountGroupRel> tabSysAccountGroupRelList = accountGroupRelService.listByUserId(userId,
                AccountGroupMemberTypeEnum.ADMIN.getCode());
        StringBuilder accountGroupIdStr = new StringBuilder();
        if (CollectionUtils.isNotEmpty(tabSysAccountGroupRelList)) {
            for (TabSysAccountGroupRel accountGroupRel : tabSysAccountGroupRelList) {
                accountGroupIdStr.append(accountGroupRel.getAccountGroupId()).append(",");
            }
        }
        String accountGroupIdResult = accountGroupIdStr.toString();
        if (StringUtils.isBlank(accountGroupIdResult)) {
            result.setAccountGroupIds("");
        } else {
            result.setAccountGroupIds(accountGroupIdResult.substring(0, accountGroupIdResult.length() - 1));
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Void> delete(Integer userId) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        //删除角色关联
        userRoleService.deleteByUserId(userId);
        //删除资源关联
        userResourceService.deleteByUserId(userId);
        //删除账号分组关联
        accountGroupRelService.deleteByUserId(userId, AccountGroupMemberTypeEnum.ADMIN.getCode());
        userService.delete(userId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> getUserByEnterpriseId(Integer enterpriseId) {
        TabSysUser tabUser = userService.getUserByEnterpriseId(enterpriseId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, tabUser));
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUserByPhoneNumber(String phoneNumber) {
        List<TabSysUser> list = this.userService.listAllUserByPhoneNumber(phoneNumber);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(UserDTO.class, list));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> listUserByPhoneNumber(String phoneNumber, Integer enterpriseId) {
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户数据为空");
        }
        List<TabSysUser> list = this.userService.listAllUserByPhoneNumber(phoneNumber);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabSysUser user : list) {
                if (enterpriseId.intValue() == user.getEnterpriseId().intValue()) {
                    return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, user));
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> login(String phoneNumber, Integer enterpriseId, String password) {
        TabSysUser user = this.userService.login(phoneNumber, enterpriseId, password);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(UserDTO.class, user));
    }

    @Override
    public ServiceResponse<Page<UserListDTO>> pageUser(UserListQO params) {
        if (params.getAccountGroupId() != null) {
            //查询分组下的管理员列表
            List<TabSysAccountGroupRel> list = accountGroupRelService.listByAccountGroupId(params.getAccountGroupId(),
                    AccountGroupMemberTypeEnum.ADMIN.getCode());
            if (CollectionUtils.isNotEmpty(list)) {
                List<Integer> userIdList = new ArrayList<>(list.size());
                for (TabSysAccountGroupRel temp : list) {
                    userIdList.add(temp.getUserId());
                }
                params.setUserIdList(userIdList);
            }
            //如果是空的，则返回空
            if (CollectionUtils.isEmpty(params.getUserIdList())) {
                return ServiceResponse.success(new Page<>());
            }

        }
        com.github.pagehelper.Page page = userService.pageUser(params);
        Page<UserListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<UserListDTO>> listUser(UserListQO params) {
        List<UserListDTO> list = userService.listUser(params);
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUserByIdList(List<Integer> userIdList) {
        List<TabSysUser> list = userService.listUserByIdList(userIdList);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UserDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<UserDetail> getUserDetail(String token) {
        String key = Constants.TOKEN_KEY + token;
        Object cache = RedisUtil.getCache(key);
        if (cache == null) {
            return ServiceResponse.success();
        }
        return ServiceResponse.success((UserDetail) cache);
    }

    @Override
    public ServiceResponse<Void> removeAccountGroup(Integer accountGroupId, Integer userId, Integer type) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        TabSysAccountGroup tabSysAccountGroup = accountGroupService.getById(accountGroupId);
        if (tabSysAccountGroup == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组ID输入有误，无此数据");
        }
        accountGroupRelService.deleteByAccountGroupIdAndUserId(userId, accountGroupId, type);
        return ServiceResponse.success();
    }

    /**
     * 保存关联数据，角色关联、资源关联
     * @Title: saveRole 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param enterpriseId
     * @param userRoleIds
     * @param userResourceIds 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    private ServiceResponse<Integer> saveRole(Integer userId, Integer enterpriseId, String userRoleIds,
            String userResourceIds, String accountGroupIds) {
        String[] userRoleIdArr;
        String[] userResourceIdArr;
        String[] accountGroupIdArr;
        ServiceResponse<String[]> userRoleValid = ValidSplitUtils.validStr(userRoleIds);
        if (userRoleValid.isSuccess()) {
            userRoleIdArr = userRoleValid.getResult();
        } else {
            return ServiceResponse.failure(userRoleValid.getCode(), userRoleValid.getMessage());
        }
        ServiceResponse<String[]> userResourceIdValid = ValidSplitUtils.validStr(userResourceIds);
        if (userResourceIdValid.isSuccess()) {
            userResourceIdArr = userResourceIdValid.getResult();
        } else {
            return ServiceResponse.failure(userResourceIdValid.getCode(), userResourceIdValid.getMessage());
        }
        //新增用户角色关联数据
        UserRoleDTO userRoleDTO = new UserRoleDTO();
        userRoleDTO.setEnterpriseId(enterpriseId);
        userRoleDTO.setUserId(userId);
        for (String userRole : userRoleIdArr) {
            int roleId = Integer.parseInt(userRole);
            TabSysRole tabSysRole = roleService.getById(roleId);
            if (tabSysRole == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色数据有误，无此角色数据");
            }
            userRoleDTO.setRoleId(roleId);
            ServiceResponse result = userRoleApiService.save(userRoleDTO);
            if (!result.isSuccess()) {
                return result;
            }
        }
        //新增用户资源关联数据
        UserResourceDTO userResourceDTO = new UserResourceDTO();
        userResourceDTO.setUserId(userId);
        userResourceDTO.setEnterpriseId(enterpriseId);
        for (String userResource : userResourceIdArr) {
            int resourceId = Integer.parseInt(userResource);
            TabSysResource tabSysResource = resourceService.getResource(resourceId);
            if (tabSysResource == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源数据有误，无此资源数据");
            }
            userResourceDTO.setResourceId(resourceId);
            ServiceResponse result = userResourceApiService.save(userResourceDTO);
            if (!result.isSuccess()) {
                return result;
            }
        }
        //账号分组用户关联数据维护
        if (StringUtils.isNotBlank(accountGroupIds)) {
            ServiceResponse<String[]> accountGroupIdValid = ValidSplitUtils.validStr(userResourceIds);
            if (accountGroupIdValid.isSuccess()) {
                accountGroupIdArr = accountGroupIdValid.getResult();
            } else {
                return ServiceResponse.failure(accountGroupIdValid.getCode(), accountGroupIdValid.getMessage());
            }
            List<AccountGroupRelDTO> accountGroupRelDTOList = new ArrayList<>();
            Date now = new Date();
            for (String accountGroup : accountGroupIdArr) {
                int accountGroupId = Integer.parseInt(accountGroup);
                TabSysAccountGroup tabSysAccountGroup = accountGroupService.getById(accountGroupId);
                if (tabSysAccountGroup == null) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组有误，无此数据");
                }
                AccountGroupRelDTO accountGroupRelDTO = new AccountGroupRelDTO();
                accountGroupRelDTO.setType(AccountGroupMemberTypeEnum.ADMIN.getCode());
                accountGroupRelDTO.setUserId(userId);
                accountGroupRelDTO.setAccountGroupId(accountGroupId);
                accountGroupRelDTO.setCreateTime(now);
                accountGroupRelDTO.setUpdateTime(now);
                accountGroupRelDTO.setEnterpriseId(enterpriseId);
                accountGroupRelDTO.setStatus(1);
                accountGroupRelDTOList.add(accountGroupRelDTO);
            }
            accountGroupRelService.insertFetch(accountGroupRelDTOList);
        }
        return ServiceResponse.success(userId);
    }

}
